<?php

namespace Drupal\solr_health_check\Controller;

use Drupal\Core\Controller\ControllerBase;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\DependencyInjection\ContainerInterface;

class HealthCheckController extends ControllerBase
{

  /**
   * Constructs a HealthCheckController object.
   */
  public function __construct()
  {
    // No need for constructor if we're not injecting services.
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container)
  {
    return new static();
  }

  /**
   * Check Solr health for all indexes.
   *
   * @return \Symfony\Component\HttpFoundation\JsonResponse
   *   The JSON response indicating status for each index.
   */
  public function check()
  {
    $statuses = [];
    $index_storage = \Drupal::entityTypeManager()->getStorage('search_api_index');
    $indexes = $index_storage->loadMultiple();

    foreach ($indexes as $index) {
      /** @var \Drupal\search_api\IndexInterface $index */
      try {
        /** @var \Drupal\search_api_solr\Plugin\search_api\backend\SearchApiSolrBackend $backend */
        $backend = $index->getServerInstance()->getBackend();
        $solr_connector = $backend->getSolrConnector();
        $ping = $solr_connector->pingCore();

        if (is_float($ping)) {
          $statuses[$index->id()] = [
            'status' => 'up',
            'response_time' => $ping
          ];
        } else {
          $statuses[$index->id()] = [
            'status' => 'down',
            'message' => 'Solr responded but with an error status.',
            'error_status' => $ping->getStatus(),
            'error_message' => $ping->getResponse()->getStatusMessage(),
          ];
        }
      } catch (\Exception $e) {
        $statuses[$index->id()] = [
          'status' => 'down',
          'message' => $e->getMessage()
        ];
      }
    }

    $response = new JsonResponse($statuses, 200);
    $response->headers->set('Cache-Control', 'no-cache, no-store, must-revalidate');
    $response->headers->set('Pragma', 'no-cache');
    $response->headers->set('Expires', '0');

    return $response;
  }

}